

set echo on

CREATE TYPE BODY promo_stats_ot AS

   STATIC FUNCTION ODCIGetInterfaces (
                   p_interfaces OUT SYS.ODCIObjectList
                   ) RETURN NUMBER IS
   BEGIN
      p_interfaces := SYS.ODCIObjectList(
                         SYS.ODCIObject ('SYS', 'ODCISTATS2')
                         );
      RETURN ODCIConst.success;
   END ODCIGetInterfaces;

   STATIC FUNCTION ODCIStatsSelectivity (
                   p_pred_info        IN  SYS.ODCIPredInfo,
                   p_selectivity      OUT NUMBER,
                   p_args             IN  SYS.ODCIArgDescList,
                   p_start            IN  VARCHAR2,
                   p_stop             IN  VARCHAR2,
                   p_promo_category   IN  VARCHAR2,
                   p_env              IN  SYS.ODCIEnv
                   ) RETURN NUMBER IS
   BEGIN

      /* Calculate selectivity of predicate... */
      SELECT (COUNT(CASE
                       WHEN UPPER(promo_category) = p_start /* contains value of predicate */
                       THEN 0
                    END) / COUNT(*)) * 100 AS selectivity
      INTO   p_selectivity
      FROM   sh.promotions;

      RETURN ODCIConst.success;
   END ODCIStatsSelectivity;

   STATIC FUNCTION ODCIStatsFunctionCost (
                   p_func_info      IN  SYS.ODCIFuncInfo,
                   p_cost           OUT SYS.ODCICost,
                   p_args           IN  SYS.ODCIArgDescList,
                   p_promo_category IN  VARCHAR2,
                   p_env            IN  SYS.ODCIEnv
                   ) RETURN NUMBER IS

      aa_io   DBMS_SQL.NUMBER_TABLE;
      aa_ela  DBMS_SQL.NUMBER_TABLE;
      v_dummy VARCHAR2(100);

      FUNCTION snap_io RETURN NUMBER IS
         v_io NUMBER;
      BEGIN
         SELECT SUM(ss.value) INTO v_io
         FROM   v$sesstat ss
         ,      v$statname sn
         WHERE  ss.statistic# = sn.statistic#
         AND    sn.name IN ('db block gets','consistent gets');
         RETURN v_io;
      END snap_io;

   BEGIN
      p_cost := SYS.ODCICost(NULL, NULL, NULL, NULL);

      /* Snap a sample execution of the function... */
      aa_io(1) := snap_io;
      aa_ela(1) := DBMS_UTILITY.GET_TIME;
      v_dummy := promo_function('tv');
      aa_ela(2) := DBMS_UTILITY.GET_TIME;
      aa_io(2) := snap_io;

      /* Calculate costs from snaps... */
      p_cost.CPUCost := 1000 * DBMS_ODCI.ESTIMATE_CPU_UNITS(
                                  (aa_ela(2) - aa_ela(1)) / 100);
      p_cost.IOCost := aa_io(2) - aa_io(1);
      p_cost.NetworkCost := 0;

      RETURN ODCIConst.success;
   END ODCIStatsFunctionCost;

END;
/

